package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.function.Consumer;

public class ExampleMain {

    public static void main(String[] args) throws Exception {
        EntityManagerFactory emf =
                Persistence.createEntityManagerFactory("example-unit");
        try {
            persistEntities(emf);
            findAllEmployeesWithTypes(emf);
            findSalaries(emf);
        } finally {
            emf.close();
        }
    }

    private static void persistEntities(EntityManagerFactory emf) throws Exception {
        System.out.println("-- Persisting entities --");
        EntityManager em = emf.createEntityManager();
        em.getTransaction().begin();
        for (Employee employee : createEmployees()) {
            em.persist(employee);
            System.out.println(employee);
        }
        em.getTransaction().commit();
    }

    private static void findAllEmployeesWithTypes(EntityManagerFactory emf) {
        System.out.println("-- finding employees with types --");
        EntityManager em = emf.createEntityManager();
        Query query = em.createQuery("SELECT e.id, e.name, "
                + "CASE TYPE(e)"
                + "WHEN FullTimeEmployee THEN 'FullTime' "
                + "WHEN PartTimeEmployee THEN 'PartTime' "
                + "WHEN ContractEmployee THEN 'Contractor' "
                + "END"
                + " FROM Employee e");
        query.getResultList()
             .forEach((Consumer<Object[]>) o ->
                     System.out.println(Arrays.toString(o)));
        em.close();
    }

    private static void findSalaries(EntityManagerFactory emf) {
        System.out.println("-- finding salaries --");
        EntityManager em = emf.createEntityManager();
        Query query = em.createQuery("SELECT e.id, e.name, "
                + "CASE "
                + "WHEN TYPE(e) = FullTimeEmployee THEN CONCAT(e.annualSalary, ' per annum') "
                + "WHEN TYPE(e) = PartTimeEmployee THEN CONCAT(e.weeklySalary, ' per week') "
                + "WHEN TYPE(e) = ContractEmployee THEN CONCAT(e.hourlyRate, ' per hour') "
                + "END"
                + " FROM Employee e");
        query.getResultList()
             .forEach((Consumer<Object[]>) o ->
                System.out.println(Arrays.toString(o)));
        em.close();
    }


    private static List<Employee> createEmployees() {
        List<Employee> list = new ArrayList<>();
        FullTimeEmployee e = new FullTimeEmployee();
        e.setName("Sara");
        e.setSalary(100000);
        list.add(e);

        e = new FullTimeEmployee();
        e.setName("Mike");
        e.setSalary(90000);
        list.add(e);

        PartTimeEmployee e2 = new PartTimeEmployee();
        e2.setName("Jon");
        e2.setWeeklySalary(900);
        list.add(e2);

        e2 = new PartTimeEmployee();
        e2.setName("Jackie");
        e2.setWeeklySalary(1200);
        list.add(e2);

        ContractEmployee e3 = new ContractEmployee();
        e3.setName("Tom");
        e3.setHourlyRate(60);
        list.add(e3);

        e3 = new ContractEmployee();
        e3.setName("Aly");
        e3.setHourlyRate(90);
        list.add(e3);
        return list;
    }
}